package tech.sbdevelopment.websiteExample.examples;

import nl.sbdeveloper.vehiclesplus.api.VehiclesPlusAPI;
import nl.sbdeveloper.vehiclesplus.api.garages.Garage;
import nl.sbdeveloper.vehiclesplus.api.vehicles.Vehicle;
import nl.sbdeveloper.vehiclesplus.storage.db.exceptions.DataStorageException;

import java.util.Optional;

public class DeleteVehicle {
    /**
     * Removes a specific vehicle from the "police" garage based on the given model name.
     * This method retrieves the police garage, searches for a vehicle with the specified model name,
     * and removes the vehicle from the garage if found.
     * It handles the case where the garage or vehicle doesn't exist and logs appropriate messages.
     *
     * @param vehicleModelName The model name of the vehicle to remove from the police garage.
     */
    public void removeVehicleFromPoliceGarage(String vehicleModelName) {
        // Retrieve the "police" garage using its name
        Optional<Garage> policeGarageOptional = VehiclesPlusAPI.getGarage("police");

        // Check if the police garage exists, and if not, print an error message and exit
        if (policeGarageOptional.isEmpty()) {
            System.err.println("Error: The 'police' garage does not exist.");
            return;
        }

        // Get the actual garage object
        Garage policeGarage = policeGarageOptional.get();

        // Search for the vehicle with the given model name within the "police" garage
        Optional<Vehicle> vehicleToRemoveOptional = policeGarage.getVehicles().stream()
                // Convert the vehicle UUID to an actual StorageVehicle object
                .map(VehiclesPlusAPI::getVehicle)
                // Filter vehicles to match the given model name
                .filter(vehicle -> vehicle != null && vehicle.getVehicleModel().getId().equals(vehicleModelName))
                // Select the first match
                .findFirst();

        // If a matching vehicle is found, attempt to remove it
        if (vehicleToRemoveOptional.isPresent()) {
            Vehicle vehicleToRemove = vehicleToRemoveOptional.get();

            try {
                // Remove the vehicle from the garage
                vehicleToRemove.remove();
                System.out.println("Vehicle with model '" + vehicleModelName + "' has been successfully removed from the police garage.");
            } catch (DataStorageException e) {
                // Handle any errors that occur during the removal process
                System.err.println("Error: Failed to remove vehicle with model '" + vehicleModelName + "' from the garage.");
                throw new RuntimeException("Error while removing vehicle", e);
            }
        } else {
            // If no matching vehicle was found, print an error message
            System.err.println("Error: No vehicle with model '" + vehicleModelName + "' found in the police garage.");
        }
    }
}
